package be.godot.util;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.net.URL;

/**
 * 
 * Code updated by Danny De Cock, originally created by Nick Liebmann. The
 * original code can be downloaded from
 * http://javadesktop.org/forums/thread.jspa?threadID=3243&tstart=0
 *  
 */

public class LoadLibraryFromJar {
	private static final String windowsLibPrefix = "";

	private final static String windowsLibSuffix = ".dll";

	private static final String linuxLibPrefix = "lib";

	private static final String linuxLibSuffix = ".so";

	private static final String sunLibPrefix = "lib";

	private static final String sunLibSuffix = ".so";

	public static String libraryName = null;

	private static String getLocalLibraryName(String libName, String prefix,
			String suffix) {
		String osname = System.getProperty("os.name");
		if (osname.indexOf("Windows") >= 0) {
			libraryName = prefix + libName;
			return (windowsLibPrefix + libraryName + windowsLibSuffix + suffix);
		} else if (osname.indexOf("Linux") >= 0) {
			libraryName = prefix + libName;
			return (linuxLibPrefix + libraryName + linuxLibSuffix + suffix);
		} else if (osname.indexOf("SunOS") >= 0) {
			libraryName = prefix + libName;
			return (sunLibPrefix + libraryName + sunLibSuffix + suffix);
		}
		return (null);
	}

	private static File copyFileFromJARToLocalFileSystem(URL urlToCopy,
			File fDestDir) {
		String strFileName = urlToCopy.getFile();
		int nLastSlashPos = strFileName.lastIndexOf('/');
		if (nLastSlashPos != -1)
			strFileName = strFileName.substring(nLastSlashPos);

		File fDestFile = new File(fDestDir, strFileName);
		System.err.println("Creating local copy of shared library: "
				+ fDestFile.getAbsolutePath());
		try {
			fDestFile.createNewFile();
			java.io.InputStream iStream = urlToCopy.openStream();
			java.io.FileOutputStream fos = new java.io.FileOutputStream(
					fDestFile);
			byte szBuffer[] = new byte[1024];

			int nRead = 0;
			while ((nRead = iStream.read(szBuffer, 0, 1024)) > 0)
				fos.write(szBuffer, 0, nRead);

			fos.close();
			iStream.close();
		} catch (IOException ex) {
			ex.printStackTrace();
			return (null);
		}
		return (fDestFile);
	}

	public static void initLocalLibrary(String libName, String prefix) {
		initLocalLibrary(libName, prefix, "", "" + System.currentTimeMillis());
	}

	public static void initLocalLibrary(String libName, String prefix,
			String pathPrefix) {
		initLocalLibrary(libName, prefix, "", pathPrefix);
	}

	public static void initLocalLibrary(String libName, String prefix,
			String suffix, String pathPrefix) {
		if (System.getProperty("java.home") != null) {
			return;
		}
		try {
			throw new Exception();
		} catch (Exception e) {
			System.err
					.println("Dummy exception thrown to know which applet loads the native library <"
							+ prefix + libName + "*" + suffix + ">");
			e.printStackTrace();
		}
		String strLibraryPath = System.getProperty("java.library.path");
		{
			File fTempDir = null;
			try {
				fTempDir = File.createTempFile("prefix", "suffix");
				if (fTempDir != null)
					fTempDir = fTempDir.getParentFile();

				fTempDir = new File(fTempDir.getAbsolutePath() + File.separator
						+ "UnsatisfiedLinkWorkAround" + File.separator
						+ pathPrefix);
				fTempDir.mkdirs();
				fTempDir.deleteOnExit();
			} catch (IOException ex) {
			}
			if (fTempDir != null) {
				URL urlSharedLibrary = LoadLibraryFromJar.class
						.getClassLoader().getResource(
								getLocalLibraryName(libName, prefix, suffix));
				File fSharedLibrary = copyFileFromJARToLocalFileSystem(
						urlSharedLibrary, fTempDir);
				fSharedLibrary.deleteOnExit();
				strLibraryPath += File.pathSeparator + "." + File.pathSeparator
						+ fTempDir.getAbsolutePath();
				System.setProperty("java.library.path", strLibraryPath);
				try {
					//DO THE DIRTYNESS on sys_paths
					Field fieldSysPath = ClassLoader.class
							.getDeclaredField("sys_paths");
					fieldSysPath.setAccessible(true);
					if (fieldSysPath != null)
						fieldSysPath.set(System.class.getClassLoader(), null);
				} catch (SecurityException ex) {
					ex.printStackTrace();
				} catch (NoSuchFieldException ex) {
					ex.printStackTrace();
				} catch (IllegalAccessException ex) {
					ex.printStackTrace();
				} catch (IllegalArgumentException ex) {
					ex.printStackTrace();
				}
			}

		}
	}
}